/*
 * Botendo
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package net.moonleay.botendo.extensions.music.components

import com.kotlindiscord.kord.extensions.commands.application.slash.PublicSlashCommandContext
import com.kotlindiscord.kord.extensions.components.forms.ModalForm
import dev.arbjerg.lavalink.protocol.v4.LoadResult
import dev.kord.common.Color
import dev.kord.common.entity.Snowflake
import dev.kord.core.entity.Guild
import dev.kord.rest.builder.component.ActionRowBuilder
import dev.kord.rest.builder.message.actionRow
import dev.kord.rest.builder.message.embed
import dev.schlaubi.lavakord.audio.Link
import dev.schlaubi.lavakord.audio.player.Player
import dev.schlaubi.lavakord.rest.loadItem
import net.moonleay.botendo.util.*

object MusicManager {
    private var musicManagerMap: MutableMap<Snowflake, GuildTrackScheduler> = mutableMapOf()

    fun getMusicManager(guild: Guild): GuildTrackScheduler? {
        return musicManagerMap[guild.id]
    }

    fun getGuildTrackScheduler(guild: Guild, player: Player): GuildTrackScheduler {
        return musicManagerMap.computeIfAbsent(guild.id) {
            GuildTrackScheduler(player)
        }
    }


    suspend fun addToQueue(
        ctx: PublicSlashCommandContext<LinkArguments, ModalForm>,
        link: Link,
        search: String
    ) {
        addToQueue(ctx, link, search, AddType.QUEUE, false)
    }

    suspend fun upsertIntoQueue(
        ctx: PublicSlashCommandContext<LinkArguments, ModalForm>,
        link: Link,
        search: String
    ) {
        addToQueue(ctx, link, search, AddType.UPSERT, false)
    }

    suspend fun addToQueue(
        ctx: PublicSlashCommandContext<LinkArguments, ModalForm>,
        link: Link,
        search: String,
        type: AddType,
        silent: Boolean
    ) {
        val player = link.player
        val gts = getGuildTrackScheduler(ctx.guild!!.asGuild(), player)
        val u = ctx.user.asUser()
        gts.addEvents()

        when (val itm = link.loadItem(search)) {
            is LoadResult.TrackLoaded -> {
                gts.queue(itm.data, type)
                if (!silent)
                    ctx.respond {
                        this.embed {
                            this.color = Color(0x52E01A)
                            this.title = "${type.s} track from link"
                            this.description = "**${itm.data.info.title}**\n*Queue*\nby ${itm.data.info.author} ;: ${
                                TimeUtil.getTimeFormatedRaw(
                                    itm.data.info.length
                                )
                            }\n" +
                                    ">>>${itm.data.info.uri}"
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                            if (itm.data.info.uri != null) {
                                this.thumbnail {
                                    this.url =
                                        "https://img.youtube.com/vi/" + UrlUtil.getYtThumbnailUrl(itm.data.info.uri!!) + "/maxresdefault.jpg"
                                }
                            }
                        }

                        this.actionRow(fun ActionRowBuilder.() {
                            components.addAll(
                                ButtonUtil.getMusicControllerButtons(
                                    player.paused,
                                    gts.repeating
                                ).components
                            )
                        })
                    }
            }
            is LoadResult.PlaylistLoaded -> {
                player.playTrack(track = itm.data.tracks.first())
                val l= itm.data.tracks.toList().reversed()
                for (partialTrack in l) {
                    gts.queue(partialTrack, type)
                }
                if (!silent)
                    ctx.respond {
                        this.embed {
                            this.color = Color(0x52E01A)
                            this.title = "${type.s} playlist from link"
                            this.description = "**${itm.data.tracks.first().info.title}**\n*${itm.data.info.name}*\nby ${itm.data.tracks.first().info.author} ;: ${
                                TimeUtil.getTimeFormatedRaw(
                                    itm.data.tracks.first().info.length
                                )
                            }\n" +
                                    ">>>${itm.data.tracks.first().info.uri}"
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                            if (itm.data.tracks.first().info.uri != null) {
                                this.thumbnail {
                                    this.url =
                                        "https://img.youtube.com/vi/" + UrlUtil.getYtThumbnailUrl(itm.data.tracks.first().info.uri!!) + "/maxresdefault.jpg"
                                }
                            }
                        }

                        this.actionRow {
                            this.components.addAll(
                                ButtonUtil.getMusicControllerButtons(
                                    player.paused,
                                    gts.repeating
                                ).components
                            )
                        }
                    }

            }
            is LoadResult.SearchResult -> {
                gts.queue(itm.data.tracks.first(), type)
                if (!silent)
                    ctx.respond {
                        this.embed {
                            this.color = Color(0x52E01A)
                            this.title = "${type.s} track from query"
                            this.description = "**${itm.data.tracks.first().info.title}**\n*Queue*\nby ${itm.data.tracks.first().info.author} ;: ${
                                TimeUtil.getTimeFormatedRaw(
                                    itm.data.tracks.first().info.length
                                )
                            }\n" +
                                    ">>>${itm.data.tracks.first().info.uri}"
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                            if (itm.data.tracks.first().info.uri != null) {
                                this.thumbnail {
                                    this.url =
                                        "https://img.youtube.com/vi/" + UrlUtil.getYtThumbnailUrl(itm.data.tracks.first().info.uri!!) + "/maxresdefault.jpg"
                                }
                            }
                        }

                        this.actionRow {
                            this.components.addAll(
                                ButtonUtil.getMusicControllerButtons(
                                    player.paused,
                                    gts.repeating
                                ).components
                            )
                        }
                    }

            }
            is LoadResult.NoMatches -> {
                if(gts.isEmpty()) {
                    player.stopTrack()
                    link.destroy()
                }
                if (!silent)
                    ctx.respond {
                        this.embed {
                            this.color = Color(0xE0311A)
                            this.title = "Not found"
                            this.description = "There were no matches."
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                        }
                    }
            }
            is LoadResult.LoadFailed -> {
                if (gts.isEmpty()) {
                    player.stopTrack()
                    link.destroy()
                }
                if (!silent)
                    ctx.respond {
                        this.embed {
                            this.color = Color(0xE0311A)
                            this.title = "Load failed"
                            this.description = "There was an error while loading."
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                        }
                    }
            }
        }
    }
    enum class AddType(val s: String) {
        QUEUE("Added"),
        UPSERT("Upserted")
    }
}

